@@ -4,33 +4,37 @@
 # Distributed under the terms of the Modified BSD License.
 
 import asyncio
-import sys
 import inspect
-from typing import Callable, Awaitable, Any, Union
+import sys
+from typing import Any, Awaitable, Callable, Optional, TypeVar, Union
 
 
 def check_ipython() -> None:
     # original from vaex/asyncio.py
     IPython = sys.modules.get('IPython')
     if IPython:
-        version_str = IPython.__version__  # type: ignore
+        version_str = IPython.__version__
         # We get rid of any trailing ".dev"
         version_str = version_str.replace(".dev", "")
 
         IPython_version = tuple(map(int, version_str.split('.')))
         if IPython_version < (7, 0, 0):
-            raise RuntimeError(f'You are using IPython {IPython.__version__} '  # type: ignore
-                               'while we require 7.0.0+, please update IPython')
+            raise RuntimeError(
+                f'You are using IPython {IPython.__version__} '
+                'while we require 7.0.0+, please update IPython'
+            )
 
 
 def check_patch_tornado() -> None:
     """If tornado is imported, add the patched asyncio.Future to its tuple of acceptable Futures"""
     # original from vaex/asyncio.py
     if 'tornado' in sys.modules:
-        import tornado.concurrent  # type: ignore
+        import tornado.concurrent
+
         if asyncio.Future not in tornado.concurrent.FUTURES:
-            tornado.concurrent.FUTURES = \
-                tornado.concurrent.FUTURES + (asyncio.Future, )  # type: ignore
+            tornado.concurrent.FUTURES = tornado.concurrent.FUTURES + (  # type: ignore
+                asyncio.Future,
+            )
 
 
 def just_run(coro: Awaitable) -> Any:
@@ -52,12 +56,16 @@ def just_run(coro: Awaitable) -> Any:
         # to have reentrant event loops
         check_ipython()
         import nest_asyncio
+
         nest_asyncio.apply()
         check_patch_tornado()
     return loop.run_until_complete(coro)
 
 
-def run_sync(coro: Callable) -> Callable:
+T = TypeVar("T")
+
+
+def run_sync(coro: Callable[..., Awaitable[T]]) -> Callable[..., T]:
     """Runs a coroutine and blocks until it has executed.
 
     An event loop is created if no one already exists. If an event loop is
@@ -74,8 +82,10 @@ def run_sync(coro: Callable) -> Callable:
     result :
         Whatever the coroutine returns.
     """
+
     def wrapped(*args, **kwargs):
         return just_run(coro(*args, **kwargs))
+
     wrapped.__doc__ = coro.__doc__
     return wrapped
 
@@ -95,3 +105,11 @@ async def ensure_async(obj: Union[Awaitable, Any]) -> Any:
         return result
     # obj doesn't need to be awaited
     return obj
+
+
+async def run_hook(hook: Optional[Callable], **kwargs: Any) -> None:
+    if hook is None:
+        return
+    res = hook(**kwargs)
+    if inspect.isawaitable(res):
+        await res
